import 'package:flutter/material.dart';
import 'package:path_stack/path_stack.dart';

import 'advanced_tab_example_pages.dart';

class AppPaths {
  static const String tabsCategory = "tabs/";
}

/// A double-nested page stack that also supports full-screen paths and back history support.

/// Has a main tab menu that wraps the main (persistent) routes. Settings contains a secondary Tab Menu that also retains its state.
/// Also supports 2 fullscreen routes (Compose and DetailsPage) that are NOT  stateful.
/// with DetailsPage showing an example of route args being parsed (/details/83 -> DetailsPage(itemId: 83))
///
class AdvancedTabExample extends StatefulWidget {
  @override
  _AdvancedTabExampleState createState() => _AdvancedTabExampleState();
}

ValueNotifier<String> pathNotifier = ValueNotifier(AppPaths.tabsCategory + HomePage.path);
GlobalKey<PathStackState> pathStackKey = GlobalKey();

/// Shows how you could implement a simple history stack by hand (allowing web-like back behavior)
List<String> history = [];
// Move the history back 1 entry
void goBack() {
  if (history.length > 1) {
    String prevPath = history[history.length - 2];
    history..removeLast()..removeLast(); // remove last 2 history entries
    pathNotifier.value = prevPath; // switch, adding a new history entry
  }
}

class _AdvancedTabExampleState extends State<AdvancedTabExample> {
  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder<String>(
      valueListenable: pathNotifier,
      builder: (_, currentPath, __) {
        /// Add entry to history each time we change path
        if (history.isEmpty || currentPath != history.last) {
          history.add(currentPath);
        }

        /// Root level path-stack, contains our main tabbed scaffold, and any full-screen routes
        return PathStack(
          path: currentPath,
          key: pathStackKey,
          entries: [
            PathStackEntry(
                path: AppPaths.tabsCategory,
                builder: (_, __) {
                  /// Main-app path-stack, wraps all childed in MainScaffold
                  return PathStack(
                    path: currentPath,
                    parentPath: AppPaths.tabsCategory,
                    // Main scaffold is wrapped here
                    scaffoldBuilder: (_, stack) => MainScaffold(child: stack),
                    transitionBuilder: (_, stack, animation) => FadeTransition(opacity: animation, child: stack),
                    entries: [
                      /// Home
                      PathStackEntry(path: HomePage.path, builder: (_, __) => HomePage("")),

                      /// Settings
                      PathStackEntry(
                        path: SettingsHome.path,
                        builder: (_, __) {
                          /// Settings PathStack, wraps all Settings children in SettingsScaffold
                          return PathStack(
                            path: currentPath,
                            parentPath: AppPaths.tabsCategory + SettingsHome.path,
                            // Settings scaffold is wrapped here
                            scaffoldBuilder: (_, child) => SettingsScaffold(child: child),
                            entries: [
                              PathStackEntry(path: ProfileSettings.path, builder: (_, __) => ProfileSettings("")),
                              PathStackEntry(path: AlertSettings.path, builder: (_, __) => AlertSettings("")),
                              PathStackEntry(path: BillingSettings.path, builder: (_, __) => BillingSettings("")),
                            ],
                          );
                        },
                      ),
                    ],
                  );
                }),

            /// Full-screen Route
            PathStackEntry(path: ComposePage.path, builder: (_, __) => ComposePage(), maintainState: false),
            PathStackEntry(
                path: DetailsPage.path,
                // Simulate an itemId, normally you'd parse this from the queryString or path suffix (TODO)
                builder: (_, args) => DetailsPage.fromArgs(args),
                maintainState: false)
          ],
        );
      },
    );
  }
}

class BackBtn extends StatelessWidget {
  Widget build(BuildContext context) => OutlinedButton(onPressed: goBack, child: Text("<< Back"));
}
